@@ -25,13 +25,17 @@ using namespace cv;
 #  include "opencl_kernels_core.hpp"
 #endif // HAVE_OPENCL
 
-#if defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL)
+#ifdef HAVE_VA_INTEL
+#ifdef HAVE_VA_INTEL_OLD_HEADER
 #  include <CL/va_ext.h>
-#endif // HAVE_VA_INTEL && HAVE_OPENCL
+#else
+#  include <CL/cl_va_api_media_sharing_intel.h>
+#endif
+#endif
 
 namespace cv { namespace va_intel {
 
-#if defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL)
+#ifdef HAVE_VA_INTEL
 
 static clGetDeviceIDsFromVA_APIMediaAdapterINTEL_fn clGetDeviceIDsFromVA_APIMediaAdapterINTEL = NULL;
 static clCreateFromVA_APIMediaSurfaceINTEL_fn       clCreateFromVA_APIMediaSurfaceINTEL       = NULL;
@@ -40,7 +44,7 @@ static clEnqueueReleaseVA_APIMediaSurfacesINTEL_fn  clEnqueueReleaseVA_APIMediaS
 
 static bool contextInitialized = false;
 
-#endif // HAVE_VA_INTEL && HAVE_OPENCL
+#endif // HAVE_VA_INTEL
 
 namespace ocl {
 
@@ -50,7 +54,7 @@ Context& initializeContextFromVA(VADisplay display, bool tryInterop)
 #if !defined(HAVE_VA)
     NO_VA_SUPPORT_ERROR;
 #else  // !HAVE_VA
-# if (defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL))
+#   ifdef HAVE_VA_INTEL
     contextInitialized = false;
     if (tryInterop)
     {
@@ -106,7 +110,7 @@ Context& initializeContextFromVA(VADisplay display, bool tryInterop)
                                                                CL_PREFERRED_DEVICES_FOR_VA_API_INTEL, 0, NULL, &numDevices);
             if ((status != CL_SUCCESS) || !(numDevices > 0))
                 continue;
-            numDevices = 1; // initializeContextFromHandle() expects only 1 device
+            numDevices = 1; // OpenCV expects only 1 device
             status = clGetDeviceIDsFromVA_APIMediaAdapterINTEL(platforms[i], CL_VA_API_DISPLAY_INTEL, display,
                                                                CL_PREFERRED_DEVICES_FOR_VA_API_INTEL, numDevices, &device, NULL);
             if (status != CL_SUCCESS)
@@ -135,12 +139,26 @@ Context& initializeContextFromVA(VADisplay display, bool tryInterop)
         if (found >= 0)
         {
             contextInitialized = true;
-            Context& ctx = Context::getDefault(false);
-            initializeContextFromHandle(ctx, platforms[found], context, device);
-            return ctx;
+
+            cl_platform_id platform = platforms[found];
+            std::string platformName = PlatformInfo(&platform).name();
+
+            OpenCLExecutionContext clExecCtx;
+            try
+            {
+                clExecCtx = OpenCLExecutionContext::create(platformName, platform, context, device);
+            }
+            catch (...)
+            {
+                clReleaseDevice(device);
+                clReleaseContext(context);
+                throw;
+            }
+            clExecCtx.bind();
+            return const_cast<Context&>(clExecCtx.getContext());
         }
     }
-# endif // HAVE_VA_INTEL && HAVE_OPENCL
+# endif // HAVE_VA_INTEL
     {
         Context& ctx = Context::getDefault(true);
         return ctx;
@@ -148,7 +166,7 @@ Context& initializeContextFromVA(VADisplay display, bool tryInterop)
 #endif  // !HAVE_VA
 }
 
-#if defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL)
+#ifdef HAVE_VA_INTEL
 static bool ocl_convert_nv12_to_bgr(cl_mem clImageY, cl_mem clImageUV, cl_mem clBuffer, int step, int cols, int rows)
 {
     ocl::Kernel k;
@@ -174,7 +192,7 @@ static bool ocl_convert_bgr_to_nv12(cl_mem clBuffer, int step, int cols, int row
     size_t globalsize[] = { (size_t)cols, (size_t)rows };
     return k.run(2, globalsize, 0, false);
 }
-#endif // HAVE_VA_INTEL && HAVE_OPENCL
+#endif // HAVE_VA_INTEL
 
 } // namespace cv::va_intel::ocl
 
@@ -497,7 +515,7 @@ void convertToVASurface(VADisplay display, InputArray src, VASurfaceID surface,
     Size srcSize = src.size();
     CV_Assert(srcSize.width == size.width && srcSize.height == size.height);
 
-# if (defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL))
+#ifdef HAVE_VA_INTEL
     if (contextInitialized)
     {
         UMat u = src.getUMat();
@@ -545,7 +563,7 @@ void convertToVASurface(VADisplay display, InputArray src, VASurfaceID surface,
             CV_Error(cv::Error::OpenCLApiCallError, "OpenCL: clReleaseMem failed (UV plane)");
     }
     else
-# endif // HAVE_VA_INTEL && HAVE_OPENCL
+# endif // HAVE_VA_INTEL
     {
         Mat m = src.getMat();
 
@@ -598,7 +616,7 @@ void convertFromVASurface(VADisplay display, VASurfaceID surface, Size size, Out
     // TODO Need to specify ACCESS_WRITE here somehow to prevent useless data copying!
     dst.create(size, dtype);
 
-# if (defined(HAVE_VA_INTEL) && defined(HAVE_OPENCL))
+#ifdef HAVE_VA_INTEL
     if (contextInitialized)
     {
         UMat u = dst.getUMat();
@@ -646,7 +664,7 @@ void convertFromVASurface(VADisplay display, VASurfaceID surface, Size size, Out
             CV_Error(cv::Error::OpenCLApiCallError, "OpenCL: clReleaseMem failed (UV plane)");
     }
     else
-# endif // HAVE_VA_INTEL && HAVE_OPENCL
+# endif // HAVE_VA_INTEL
     {
         Mat m = dst.getMat();
 
