@@ -16,7 +16,7 @@ namespace cv { namespace dnn { namespace cuda4dnn { namespace kernels {
     void region(const csl::Stream& stream, csl::Span<T> output, csl::View<T> input, csl::View<T> bias,
         T object_prob_cutoff, T class_prob_cutoff,
         std::size_t boxes_per_cell, std::size_t box_size,
-        std::size_t rows, std::size_t cols,
+        std::size_t rows, std::size_t cols, T scale_x_y,
         std::size_t height_norm, std::size_t width_norm,
         bool if_true_sigmoid_else_softmax);
 
