@@ -17,6 +17,9 @@ const static std::string IMAGE_BIKES = "detectors_descriptors_evaluation/images_
  * Descriptors's rotation invariance check
  */
 
+INSTANTIATE_TEST_CASE_P(SIFT, DescriptorRotationInvariance,
+                        Value(IMAGE_TSUKUBA, SIFT::create(), SIFT::create(), 0.98f));
+
 INSTANTIATE_TEST_CASE_P(BRISK, DescriptorRotationInvariance,
                         Value(IMAGE_TSUKUBA, BRISK::create(), BRISK::create(), 0.99f));
 
@@ -33,6 +36,9 @@ INSTANTIATE_TEST_CASE_P(AKAZE_DESCRIPTOR_KAZE, DescriptorRotationInvariance,
  * Descriptor's scale invariance check
  */
 
+INSTANTIATE_TEST_CASE_P(SIFT, DescriptorScaleInvariance,
+                        Value(IMAGE_BIKES, SIFT::create(0, 3, 0.09), SIFT::create(0, 3, 0.09), 0.78f));
+
 INSTANTIATE_TEST_CASE_P(AKAZE, DescriptorScaleInvariance,
                         Value(IMAGE_BIKES, AKAZE::create(), AKAZE::create(), 0.6f));
 
