@@ -10,12 +10,11 @@
 
 namespace
 {
-#define CORE_FLUID [] () { return cv::compile_args(cv::gapi::core::fluid::kernels()); }
+#define CORE_FLUID [] () { return cv::compile_args(cv::gapi::use_only{cv::gapi::core::fluid::kernels()}); }
 }  // anonymous namespace
 
 namespace opencv_test
 {
-
 // FIXME: Windows accuracy problems after recent update!
 INSTANTIATE_TEST_CASE_P(MathOpTestFluid, MathOpTest,
                         Combine(Values(CV_8UC3, CV_8UC1, CV_16SC1, CV_32FC1),
@@ -25,11 +24,46 @@ INSTANTIATE_TEST_CASE_P(MathOpTestFluid, MathOpTest,
                                        cv::Size(128, 128)),
                                 Values(-1, CV_8U, CV_32F),
                                 Values(CORE_FLUID),
-                                Values(ADD, SUB, DIV, MUL),
+                                Values(DIV, MUL),
                                 testing::Bool(),
                                 Values(1.0),
                                 testing::Bool()));
 
+// FIXME: Accuracy test for SUB math operation fails on FullHD and HD CV_16SC1 input cv::Mat,
+//        double-presicion input cv::Scalar and CV_32FC1 output cv::Mat on Mac.
+//        Accuracy test for ADD math operation fails on HD CV_16SC1 input cv::Mat,
+//        double-presicion input cv::Scalar and CV_32FC1 output cv::Mat on Mac.
+//        As failures are sporadic, disabling all instantiation cases for SUB and ADD.
+//        Github ticket: https://github.com/opencv/opencv/issues/18373.
+INSTANTIATE_TEST_CASE_P(DISABLED_MathOpTestFluid, MathOpTest,
+                        Combine(Values(CV_8UC3, CV_8UC1, CV_16SC1, CV_32FC1),
+                                Values(cv::Size(1920, 1080),
+                                       cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1, CV_8U, CV_32F),
+                                Values(CORE_FLUID),
+                                Values(ADD, SUB),
+                                testing::Bool(),
+                                Values(1.0),
+                                testing::Bool()));
+
+// FIXME: Accuracy test for SUB math operation fails on CV_16SC1 input cv::Mat, double-presicion
+//        input cv::Scalar and CV_32FC1 output cv::Mat on Mac.
+//        As failures are sporadic, disabling all instantiation cases for SUB operation.
+//        Github ticket: https://github.com/opencv/opencv/issues/18373.
+INSTANTIATE_TEST_CASE_P(DISABLED_SubTestFluid, MathOpTest,
+                        Combine(Values(CV_8UC1, CV_16SC1 , CV_32FC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1, CV_8U, CV_32F),
+                                Values(CORE_FLUID),
+                                Values(SUB),
+                                testing::Bool(),
+                                Values (1.0),
+                                testing::Bool()));
+
 INSTANTIATE_TEST_CASE_P(MulSTestFluid, MulDoubleTest,
                         Combine(Values(CV_8UC1, CV_16SC1, CV_32FC1),
                                 Values(cv::Size(1280, 720),
@@ -46,6 +80,22 @@ INSTANTIATE_TEST_CASE_P(DivCTestFluid, DivCTest,
                                 Values(CV_8U, CV_32F),
                                 Values(CORE_FLUID)));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_MeanTestFluid, MeanTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
+INSTANTIATE_TEST_CASE_P(MaskTestFluid, MaskTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
 INSTANTIATE_TEST_CASE_P(AbsDiffTestFluid, AbsDiffTest,
                         Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
                                 Values(cv::Size(1280, 720),
@@ -70,7 +120,8 @@ INSTANTIATE_TEST_CASE_P(BitwiseTestFluid, BitwiseTest,
                                        cv::Size(128, 128)),
                                 Values(-1),
                                 Values(CORE_FLUID),
-                                Values(AND, OR, XOR)));
+                                Values(AND, OR, XOR),
+                                testing::Bool()));
 
 INSTANTIATE_TEST_CASE_P(BitwiseNotTestFluid, NotTest,
                         Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_16SC1),
@@ -99,6 +150,15 @@ INSTANTIATE_TEST_CASE_P(MaxTestFluid, MaxTest,
                                 Values(-1),
                                 Values(CORE_FLUID)));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_SumTestFluid, SumTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(AbsToleranceScalar(1e-5).to_compare_obj())));
+
 INSTANTIATE_TEST_CASE_P(CompareTestFluid, CmpTest,
                         Combine(Values(CV_8UC3, CV_8UC1, CV_16SC1, CV_32FC1),
                                 Values(cv::Size(1920, 1080),
@@ -108,7 +168,21 @@ INSTANTIATE_TEST_CASE_P(CompareTestFluid, CmpTest,
                                 Values(CV_8U),
                                 Values(CORE_FLUID),
                                 Values(CMP_EQ, CMP_GE, CMP_NE, CMP_GT, CMP_LT, CMP_LE),
-                                testing::Bool()));
+                                Values(false),
+                                Values(AbsExact().to_compare_obj())));
+
+// FIXME: solve comparison error to unite with the test above
+INSTANTIATE_TEST_CASE_P(CompareTestFluidScalar, CmpTest,
+                        Combine(Values(CV_8UC3, CV_8UC1, CV_16SC1, CV_32FC1),
+                                Values(cv::Size(1920, 1080),
+                                       cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(CV_8U),
+                                Values(CORE_FLUID),
+                                Values(CMP_EQ, CMP_GE, CMP_NE, CMP_GT, CMP_LT, CMP_LE),
+                                Values(true),
+                                Values(AbsSimilarPoints(1, 0.01).to_compare_obj())));
 
 INSTANTIATE_TEST_CASE_P(AddWeightedTestFluid, AddWeightedTest,
                         Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
@@ -119,6 +193,24 @@ INSTANTIATE_TEST_CASE_P(AddWeightedTestFluid, AddWeightedTest,
                                 Values(CORE_FLUID),
                                 Values(Tolerance_FloatRel_IntAbs(1e-5, 2).to_compare_obj())));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_NormTestFluid, NormTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(AbsToleranceScalar(1e-5).to_compare_obj()),
+                                Values(NORM_INF, NORM_L1, NORM_L2)));
+
+INSTANTIATE_TEST_CASE_P(DISABLED_IntegralTestFluid, IntegralTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
 INSTANTIATE_TEST_CASE_P(LUTTestFluid, LUTTest,
                         Combine(Values(CV_8UC1, CV_8UC3),
                                 Values(cv::Size(1920, 1080),
@@ -129,17 +221,32 @@ INSTANTIATE_TEST_CASE_P(LUTTestFluid, LUTTest,
                                 Values(CORE_FLUID)));
 
 INSTANTIATE_TEST_CASE_P(ConvertToFluid, ConvertToTest,
-                        Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_32FC1),
-                                Values(cv::Size(1920, 1080),
-                                       cv::Size(1280, 720),
+                        Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_16SC1, CV_32FC1),
+                                Values(cv::Size(1280, 720),
                                        cv::Size(640, 480),
                                        cv::Size(128, 128)),
-                                Values(CV_8U, CV_16U, CV_32F),
+                                Values(CV_8U, CV_16U, CV_16S, CV_32F),
                                 Values(CORE_FLUID),
                                 Values(Tolerance_FloatRel_IntAbs(1e-5, 2).to_compare_obj()),
                                 Values(2.5, 1.0, -1.0),
                                 Values(250.0, 0.0, -128.0)));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_ConcatHorTestFluid, ConcatHorTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
+INSTANTIATE_TEST_CASE_P(DISABLED_ConcatVertTestFluid, ConcatVertTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
 INSTANTIATE_TEST_CASE_P(Split3TestFluid, Split3Test,
                         Combine(Values(CV_8UC3),
                                 Values(cv::Size(1280, 720),
@@ -174,6 +281,32 @@ INSTANTIATE_TEST_CASE_P(Merge4TestFluid, Merge4Test,
                                 Values(CV_8UC4),
                                 Values(CORE_FLUID)));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_RemapTestFluid, RemapTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID)));
+
+INSTANTIATE_TEST_CASE_P(DISABLED_FlipTestFluid, FlipTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(0,1,-1)));
+
+INSTANTIATE_TEST_CASE_P(DISABLED_CropTestFluid, CropTest,
+                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(cv::Rect(10, 8, 20, 35), cv::Rect(4, 10, 37, 50))));
+
 INSTANTIATE_TEST_CASE_P(SelectTestFluid, SelectTest,
                         Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_16SC1),
                                 Values(cv::Size(1920, 1080),
@@ -231,6 +364,15 @@ INSTANTIATE_TEST_CASE_P(ThresholdTestFluid, ThresholdTest,
                                        cv::Scalar(100, 100, 100, 100),
                                        cv::Scalar(255, 255, 255, 255))));
 
+INSTANTIATE_TEST_CASE_P(DISABLED_ThresholdTestFluid, ThresholdOTTest,
+                        Combine(Values(CV_8UC1),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(cv::THRESH_OTSU, cv::THRESH_TRIANGLE)));
+
 INSTANTIATE_TEST_CASE_P(InRangeTestFluid, InRangeTest,
                         Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_16SC1, CV_32FC1),
                                 Values(cv::Size(1920, 1080),
@@ -257,6 +399,20 @@ INSTANTIATE_TEST_CASE_P(ResizeTestFluid, ResizeTest,
                                        cv::Size(64, 64),
                                        cv::Size(30, 30))));
 
+INSTANTIATE_TEST_CASE_P(ResizeTestFxFyFluid, ResizeTestFxFy,
+                        Combine(Values(CV_8UC3/*CV_8UC1, CV_16UC1, CV_16SC1*/),
+                                Values(cv::Size(1280, 720),
+                                       cv::Size(640, 480),
+                                       cv::Size(128, 128),
+                                       cv::Size(64, 64),
+                                       cv::Size(30, 30)),
+                                Values(-1),
+                                Values(CORE_FLUID),
+                                Values(AbsExact().to_compare_obj()),
+                                Values(/*cv::INTER_NEAREST,*/ cv::INTER_LINEAR/*, cv::INTER_AREA*/),
+                                Values(0.5, 1, 2),
+                                Values(0.5, 1, 2)));
+
 INSTANTIATE_TEST_CASE_P(BackendOutputAllocationTestFluid, BackendOutputAllocationTest,
                         Combine(Values(CV_8UC3, CV_16SC2, CV_32FC1),
                                 Values(cv::Size(50, 50)),
@@ -277,268 +433,4 @@ INSTANTIATE_TEST_CASE_P(ReInitOutTestFluid, ReInitOutTest,
                                 Values(CORE_FLUID),
                                 Values(cv::Size(640, 400),
                                        cv::Size(10, 480))));
-
-INSTANTIATE_TEST_CASE_P(CopyTestFluid, CopyTest,
-                        Combine(Values( CV_8UC1, CV_8UC3, CV_16UC1, CV_16SC1, CV_32FC1 ),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(-1),
-                                Values(CORE_FLUID)));
-
-//----------------------------------------------------------------------
-// FIXME: Clean-up test configurations which are enabled already
-#if 0
-INSTANTIATE_TEST_CASE_P(MathOpTestCPU, MathOpTest,
-                        Combine(Values(ADD, DIV, MUL),
-                                testing::Bool(),
-                                Values(CV_8UC1, CV_16SC1, CV_32FC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(-1, CV_8U, CV_32F),
-/*init output matrices or not*/ testing::Bool(),
-                                Values(false)));
-
-INSTANTIATE_TEST_CASE_P(SubTestCPU, MathOpTest,
-                        Combine(Values(SUB),
-                                testing::Bool(),
-                                Values(CV_8UC1, CV_16SC1 , CV_32FC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(-1, CV_8U, CV_32F),
-/*init output matrices or not*/ testing::Bool(),
-                                testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(MulSTestCPU, MulSTest,
-                        Combine(Values(CV_8UC1, CV_16SC1, CV_32FC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(DivCTestCPU, DivCTest,
-                        Combine(Values(CV_8UC1, CV_16SC1, CV_32FC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(CV_8U, CV_32F),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(MeanTestCPU, MeanTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(SelectTestCPU, SelectTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(Polar2CartCPU, Polar2CartTest,
-                        Combine(Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(Cart2PolarCPU, Cart2PolarTest,
-                        Combine(Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(CompareTestCPU, CmpTest,
-                        Combine(Values(CMP_EQ, CMP_GE, CMP_NE, CMP_GT, CMP_LT, CMP_LE),
-                                testing::Bool(),
-                                Values(CV_8UC1, CV_16SC1, CV_32FC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(BitwiseTestCPU, BitwiseTest,
-                        Combine(Values(AND, OR, XOR),
-                                Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(BitwiseNotTestCPU, NotTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
- /*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(MinTestCPU, MinTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(MaxTestCPU, MaxTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(SumTestCPU, SumTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool())
-                                Values(0.0),
-                       );
-
-INSTANTIATE_TEST_CASE_P(AbsDiffTestCPU, AbsDiffTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(AbsDiffCTestCPU, AbsDiffCTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(AddWeightedTestCPU, AddWeightedTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(-1, CV_8U, CV_32F),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(NormTestCPU, NormTest,
-                        Combine(Values(NORM_INF, NORM_L1, NORM_L2),
-                                Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))),
-                                Values(0.0));
-
-INSTANTIATE_TEST_CASE_P(IntegralTestCPU, IntegralTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(ThresholdTestCPU, ThresholdTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(cv::THRESH_BINARY, cv::THRESH_BINARY_INV, cv::THRESH_TRUNC, cv::THRESH_TOZERO, cv::THRESH_TOZERO_INV),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(ThresholdTestCPU, ThresholdOTTest,
-                        Combine(Values(CV_8UC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-                                Values(cv::THRESH_OTSU, cv::THRESH_TRIANGLE),
-/*init output matrices or not*/ testing::Bool()));
-
-
-INSTANTIATE_TEST_CASE_P(InRangeTestCPU, InRangeTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(Split3TestCPU, Split3Test,
-                        (Values(cv::Size(1280, 720),
-                                cv::Size(640, 480),
-                                cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(Split4TestCPU, Split4Test,
-                        (Values(cv::Size(1280, 720),
-                                cv::Size(640, 480),
-                                cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(Merge3TestCPU, Merge3Test,
-                        (Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(Merge4TestCPU, Merge4Test,
-                        (Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(RemapTestCPU, RemapTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(FlipTestCPU, FlipTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(0,1,-1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(CropTestCPU, CropTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Rect(10, 8, 20, 35), cv::Rect(4, 10, 37, 50)),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ testing::Bool()));
-
-INSTANTIATE_TEST_CASE_P(LUTTestCPU, LUTTest,
-                        Combine(Values(CV_8UC1, CV_8UC3),
-                                Values(CV_8UC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ Values(true)));
-
-INSTANTIATE_TEST_CASE_P(LUTTestCustomCPU, LUTTest,
-                        Combine(Values(CV_8UC3),
-                                Values(CV_8UC3),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128)),
-/*init output matrices or not*/ Values(true)));
-
-// PLEASE DO NOT PUT NEW ACCURACY TESTS BELOW THIS POINT! //////////////////////
-
-INSTANTIATE_TEST_CASE_P(ConvertToCPU, ConvertToTest,
-                        Combine(Values(CV_8UC3, CV_8UC1, CV_16UC1, CV_32FC1),
-                                Values(CV_8U, CV_16U, CV_32F),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-
-INSTANTIATE_TEST_CASE_P(ConcatHorTestCPU, ConcatHorTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-INSTANTIATE_TEST_CASE_P(ConcatVertTestCPU, ConcatVertTest,
-                        Combine(Values(CV_8UC1, CV_16UC1, CV_16SC1),
-                                Values(cv::Size(1280, 720),
-                                       cv::Size(640, 480),
-                                       cv::Size(128, 128))));
-
-//----------------------------------------------------------------------
-#endif // 0
-
 }
